(ns clojsa.views
  (:require [hiccup.core :refer [html]]
            [hiccup.page :refer [html5 include-css include-js]]
            [clojure.pprint]
            [ring.util.anti-forgery :refer [anti-forgery-field]]))


(defn login-form [next loggedin]
  (if loggedin
    [:a.navbar-item "Logout"]
    [:div.navbar-item.has-dropdown.is-hoverable
     [:a.navbar-link "Login"]
     [:div.navbar-dropdown.is-right
      [:div.navbar-item
       [:form#login-form {:action "/login" :method "post"}
        (anti-forgery-field)
        [:input {:type "hidden" :name "next"
                 :value (apply format next)}]
        [:div.field
         [:label.label {:for "username"} "username"]
         [:div.control
          [:input#username.input {:name "username" :type "text"
                                  :placeholder "enter your username"}]]]
        [:div.field
         [:label.label {:for "password"} "password"]
         [:div.control
          [:input#password.input {:name "password" :type "password"
                                  :placeholder "enter your password"}]]]
        [:div.field
         [:div.control
          [:button.button.is-primary "Submit"]]]]]]]))

(defn header-fragment [login]
  [:nav.navbar.is-transparent
   [:div.navbar-brand
    [:a.burger.navbar-burger {:role "button" :data-target "mainNav"}
     (repeat 3 [:span {:aria-hidden "true"}])]]
   [:div#mainNav.navbar-menu
    [:div.navbar-start
     [:a.navbar-item {:href "/bookmarks"} "Bookmarks"]]
    [:div.navbar-end
     login]]])

(defn main-template [opts header & insert-body-here]
  (html5
   {:lang "de"}
   [:head
    [:meta {:charset "utf-8"}]
    [:meta {:http-equiv "x-ua-compatible" :content "ie=edge"}]
    [:meta {:name "viewport" :content "width=device-width, initial-scale=1"}]
    [:title (get opts :title "clojsa")]
    (include-css "/css/bulma.min.css")
    (include-css "/css/style.css")
    (include-js "/js/htmx.min.js")]
   [:body
    {:hx-boost "false"}
    [:header
     header]
    [:main
     insert-body-here]
    (include-js "/js/main.js")]))

(defn index-page [req]
  (main-template
   {:title "Index"}
   [:div.container
    [:pre.output
     [:code (with-out-str (clojure.pprint/pprint req))]]]))

(defn login-page [login]
  [:div.container.box
   login])

(defn paginate [base cur last]
  (let [page-fstring "%s?page=%d"
        href (partial format page-fstring base)]
    [:section
     [:nav.container.box.pagination {:hx-boot "false"}
      [:a.pagination-previous
       {:href (href (dec cur))} "<"]
      [:a.pagination-next
       {:href (href (inc cur))} ">"]
      [:ul.pagination-list
       [:li
        [:a.pagination-link
         {:href (href 1)} (str 1)]]
       [:li
        [:span.pagination-ellipsis "&hellip;"]]
       (for [i (range (- cur 2) (+ cur 3))]
         [:li
          [:a.pagination-link
           {:href (href i)
            :class (when (= i cur) "is-current")} (str i)]])
       [:li
        [:span.pagination-ellipsis "&hellip;"]]
       [:li
        [:a.pagination-link
         {:href (href last)} (str last)]]]]]))

(defn thread-page [{:keys [title content]}]
  (list
   [:section.title [:div.container
                    [:h1.is-size-3.mb-4 title]]]
   [:section.thread
    (for [post content]
      [:article.container.box
       [:div.tile.is-ancestor
        [:aside.userinfo.card.is-shadowless.tile.is-3.is-parent
         [:div.card-content.px-0.py-0
          (let [{:keys [author regdate avatar avatar-title]} (:ui post)]
            (list
             [:div.media
              (when avatar
                [:div.media-left.is-hidden-tablet
                 [:figure.image.is-64x64 avatar]])
              [:div.media-content
               [:p.author.title.is-4 author]
               [:p.regdate.subtitle.is-6 regdate]]]
             [:div.is-hidden-mobile
              avatar-title]))]]
        [:main.postbody.tile.is-9.is-parent.is-vertical
         [:div.content.is-size-6.tile.is-child
          (:pb post)]
         [:div.level.is-mobile.tile.is-child.is-12
          [:div.level-right
           [:span.postdate.level-item
            (:pd post)]]]]]])]))

(defn bookmarks-page [beems]
  (let [{:keys [title page page-count content]} beems]
    (main-template
     {:title title}
     [:div.container
      [:h1.is-size-3.mb-4 title]]
     [:section.bookmarks
      [:nav.container.box.panel
       (for [bm content]
         [:a.panel-block {:href (str "/thread/" (:id bm))}
          (:title bm)])]]
     [:section
      (paginate (str "/bookmarks") page page-count)])))
